@@ -566,13 +566,18 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             def rwx(mode):
                 return "".join( [ ((mode& (1<<i)) and "xwr"[i%3] or "-") for i in range(8,-1,-1) ] )
             def nice(deltat):
-                # singular,plural 
-                times = _("second,seconds:minute,minutes:hour,hours:day,days:week,weeks:month,months:year,years").split(":")
-                d = abs(int(deltat))
-                for div, time in zip((60,60,24,7,4,12,100), times):
-                    if d < div * 5:
-                        return "%s%i %s" % (deltat<0 and "-" or "", d, time.split(",")[d != 1])
-                    d /= div
+                times = (
+                    (60, lambda n: ngettext("%i second","%i seconds",n)),
+                    (60, lambda n: ngettext("%i minute","%i minutes",n)),
+                    (24, lambda n: ngettext("%i hour","%i hours",n)),
+                    ( 7, lambda n: ngettext("%i day","%i days",n)),
+                    ( 4, lambda n: ngettext("%i week","%i weeks",n)),
+                    (12, lambda n: ngettext("%i month","%i months",n)),
+                    (100,lambda n: ngettext("%i year","%i years",n)) )
+                for units, msg in times:
+                    if abs(int(deltat)) < 5 * units:
+                        return msg(int(deltat)) % int(deltat)
+                    deltat /= units
             file = self.model.value_path( self.model.get_iter(paths[0]), pane )
             try:
                 stat = os.stat(file)
