@@ -23,6 +23,7 @@ import re
 import sys
 import tempfile
 import difflib
+import struct
 
 import pango
 import gobject
@@ -168,6 +169,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 self.parent.save()
             def on_save_as_activate(self, menuitem):
                 self.parent.save_file( self.pane, 1)
+            def on_make_patch_activate(self, menuitem):
+                self.parent.make_patch( self.pane )
             def on_cut_activate(self, menuitem):
                 self.parent.on_cut_activate()
             def on_copy_activate(self, menuitem):
@@ -217,8 +220,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     def _update_cursor_status(self, buf):
         def update():
             iter = buf.get_iter_at_mark( buf.get_insert() )
-            status = "%s : %s" % ( _("INS,OVR").split(",")[ self.textview_overwrite ], #insert/overwrite
-                                   _("Ln %i, Col %i") % (iter.get_line()+1, iter.get_line_offset()+1) ) #line/column
+            # Abbreviation for insert,overwrite so that it will fit in the status bar
+            insert_overwrite = _("INS,OVR").split(",")[ self.textview_overwrite ]
+            # Abbreviation for line, column so that it will fit in the status bar
+            line_column = _("Ln %i, Col %i") % (iter.get_line()+1, iter.get_line_offset()+1)
+            status = "%s : %s" % ( insert_overwrite, line_column )
             self.emit("status-changed", status  )
             raise StopIteration; yield 0
         self.scheduler.add_task( update().next )
@@ -596,7 +602,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 except (IOError, LookupError), e:
                     buffers[i].set_text("\n")
                     misc.run_dialog(
-                        _("Could not open '%s' for reading.\n\nThe error was:\n%s") % (f, str(e)),
+                        "%s\n\n%s\n%s" % (
+                            _("Could not read from '%s'") % f,
+                            _("The error was:"),
+                            str(e)),
                         parent = self)
             else:
                 panetext[i] = buffers[i].get_text( buffers[i].get_start_iter(), buffers[i].get_end_iter() )
@@ -605,6 +614,14 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             for t in tasks[:]:
                 try:
                     nextbit = t.file.read(4096)
+                    if nextbit.find("\x00") != -1:
+                        misc.run_dialog(
+                            "%s\n\n%s" % (
+                                _("Could not read from '%s'") % t.filename,
+                                _("It contains ascii nulls.\nPerhaps it is a binary file.") ),
+                                parent = self )
+                        t.buf.delete( t.buf.get_start_iter(), t.buf.get_end_iter() )
+                        tasks.remove(t)
                 except ValueError, err:
                     t.codec.pop(0)
                     if len(t.codec):
@@ -615,12 +632,17 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                         print "codec error fallback", err
                         t.buf.delete( t.buf.get_start_iter(), t.buf.get_end_iter() )
                         misc.run_dialog(
-                            _("Could not read from '%s'.\n\nI tried encodings %s.") % (t.filename, try_codecs),
+                            "%s\n\n%s" % (
+                                _("Could not read from '%s'") % t.filename,
+                                _("I tried encodings %s.") % try_codecs ),
                             parent = self)
                         tasks.remove(t)
                 except IOError, ioerr:
                     misc.run_dialog(
-                        _("Could not read from '%s'.\n\nThe error was:\n%s") % (t.filename, str(ioerr)),
+                        "%s\n\n%s\n%s" % (
+                            _("Could not read from '%s'") % t.filename,
+                            _("The error was:"),
+                            str(ioerr)),
                         parent = self)
                     tasks.remove(t)
                 else:
@@ -691,8 +713,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                         for b, t, s, l in zip(bufs, tags, starts, (c[2],c[4])):
                             b.apply_tag(t, s, b.get_iter_at_line(l))
                         if 1:
-                            text1 = "\n".join( self._get_texts(raw=1)[1  ][c[1]:c[2]] )
-                            textn = "\n".join( self._get_texts(raw=1)[i*2][c[3]:c[4]] )
+                            text1 = "\n".join( self._get_texts(raw=1)[1  ][c[1]:c[2]] ).encode("utf16")
+                            text1 = struct.unpack("%iH"%(len(text1)/2), text1)[1:]
+                            textn = "\n".join( self._get_texts(raw=1)[i*2][c[3]:c[4]] ).encode("utf16")
+                            textn = struct.unpack("%iH"%(len(textn)/2), textn)[1:]
                             matcher = difflib.SequenceMatcher(None, text1, textn)
                             #print "<<<\n%s\n---\n%s\n>>>" % (text1, textn)
                             tags = [b.get_tag_table().lookup("inline line") for b in bufs]
@@ -711,28 +735,49 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                             yield 1
         yield 1
         
+    def _get_filename_for_saving(self, title ):
+        #dialog = gtk.FileChooserDialog( ## try
+        #parent = self.widget.get_toplevel(), 
+        #action = gtk.FILE_CHOOSER_ACTION_SAVE,
+        #buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK) )
+        fselect = gtk.FileSelection( title )
+        fselect.set_transient_for(self.widget.get_toplevel() )
+        response = fselect.run()
+        if response != gtk.RESPONSE_OK:
+            fselect.destroy()
+            return None
+        else:
+            filename = fselect.get_filename()
+            fselect.destroy()
+            if os.path.exists(filename):
+                response = misc.run_dialog(
+                    _('"%s" exists!\nOverwrite?') % os.path.basename(filename),
+                    parent = self,
+                    buttonstype = gtk.BUTTONS_YES_NO)
+                if response == gtk.RESPONSE_NO:
+                    return None
+            return filename
+
+    def _save_text_to_filename(self, filename, text):
+        try:
+            open(filename, "w").write(text)
+        except IOError, e:
+            misc.run_dialog(
+                _("Error writing to %s\n\n%s.") % (filename, e),
+                self, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK)
+            return False
+        return True
+
     def save_file(self, pane, saveas=0):
         buf = self.textview[pane].get_buffer()
         bufdata = self.bufferdata[pane]
         if saveas or not bufdata.filename:
-            fselect = gtk.FileSelection( _("Choose a name for buffer %i.") % (pane+1))
-            fselect.set_transient_for(self.widget.get_toplevel() )
-            response = fselect.run()
-            if response != gtk.RESPONSE_OK:
-                fselect.destroy()
-                return melddoc.RESULT_ERROR
-            else:
-                filename = fselect.get_filename()
-                fselect.destroy()
-                if os.path.exists(filename):
-                    response = misc.run_dialog(
-                        _('"%s" exists!\nOverwrite?') % os.path.basename(filename),
-                        parent = self,
-                        buttonstype = gtk.BUTTONS_YES_NO)
-                    if response == gtk.RESPONSE_NO:
-                        return melddoc.RESULT_ERROR
+            filename = self._get_filename_for_saving( _("Choose a name for buffer %i.") % (pane+1) )
+            if filename:
                 bufdata.filename = os.path.abspath(filename)
                 self.fileentry[pane].set_filename( bufdata.filename)
+            else:
+                return melddoc.RESULT_ERROR
         text = buf.get_text(buf.get_start_iter(), buf.get_end_iter(), 0)
         if bufdata.newlines:
             if type(bufdata.newlines) == type(""):
@@ -759,18 +804,41 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     _("'%s' contains characters not encodable with '%s'\nWould you like to save as UTF-8?") % (bufdata.filename, bufdata.encoding),
                     self, gtk.MESSAGE_ERROR, gtk.BUTTONS_YES_NO) != gtk.RESPONSE_YES:
                     return melddoc.RESULT_ERROR
-        try:
-            open(bufdata.filename, "w").write(text)
-        except IOError, e:
-            misc.run_dialog(
-                _("Error writing to %s\n\n%s.") % (bufdata.filename, e),
-                self, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK)
-            return melddoc.RESULT_ERROR
-        else:
+        if self._save_text_to_filename(bufdata.filename, text):
             self.emit("file-changed", bufdata.filename)
             self.undosequence.clear()
             self.set_buffer_modified(buf, 0)
-        return melddoc.RESULT_OK
+            return melddoc.RESULT_OK
+        else:
+            return melddoc.RESULT_ERROR
+
+    def make_patch(self, pane):
+        fontdesc = pango.FontDescription(self.prefs.get_current_font())
+        dialog = gnomeglade.Component( paths.share_dir("glade2/filediff.glade"), "patchdialog")
+        dialog.widget.set_transient_for( self.widget.get_toplevel() )
+        bufs = [t.get_buffer() for t in self.textview]
+        texts = [b.get_text(*b.get_bounds()).split("\n") for b in bufs]
+        texts[0] = [l+"\n" for l in texts[0]]
+        texts[1] = [l+"\n" for l in texts[1]]
+        names = [self._get_filename(i) for i in range(2)]
+        dialog.textview.modify_font(fontdesc)
+        buf = dialog.textview.get_buffer()
+        lines = []
+        for line in difflib.unified_diff(texts[0], texts[1], names[0], names[1]):
+            buf.insert( buf.get_end_iter(), line )
+            lines.append(line)
+        result = dialog.widget.run()
+        dialog.widget.destroy()
+        if result >= 0:
+            txt = "".join(lines)
+            if result == 1: # copy
+                clip = gtk.clipboard_get()
+                clip.set_text(txt)
+                clip.store()
+            else:# save as
+                filename = self._get_filename_for_saving( _("Save patch as...") )
+                if filename:
+                    self._save_text_to_filename(filename, txt)
 
     def set_buffer_writable(self, buf, yesno):
         pane = self.textview.index(buf.textview)
