@@ -1,4 +1,4 @@
-### Copyright (C) 2002-2003 Stephen Kennedy <stevek@gnome.org>
+### Copyright (C) 2002-2004 Stephen Kennedy <stevek@gnome.org>
 
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -37,13 +37,15 @@ import task
 # optional
 sourceview_available = 0
 
-try:
-    import sourceview
-    sourceview_available = 1
-except ImportError:
-    pass
+for sourceview in "gtksourceview sourceview".split():
+    try:
+        __import__(sourceview)
+        sourceview_available = 1
+        break
+    except ImportError:
+        pass
 
-version = "0.9.4.1"
+version = "1.0.0"
 
 # magic developer switch, changes some behaviour
 developer = 0
@@ -61,15 +63,12 @@ class NewDocDialog(gnomeglade.Component):
     def __init__(self, parentapp, type):
         self.parentapp = parentapp
         gnomeglade.Component.__init__(self, paths.share_dir("glade2/meldapp.glade"), "newdialog")
-        self._map_widgets_into_lists( ("fileentry", "direntry", "cvsentry", "svnentry", "three_way_compare") )
+        self._map_widgets_into_lists( ("fileentry", "direntry", "cvsentry", "svnentry", "three_way_compare", "tablabel") )
         self.entrylists = self.fileentry, self.direntry, self.cvsentry, self.svnentry
         self.widget.set_transient_for(parentapp.widget)
         cur_page = type // 2
         self.notebook.set_current_page( cur_page )
-        if cur_page < 2:
-            self.three_way_compare[cur_page].set_active( (type+1) % 2 )
-            self.three_way_compare[cur_page].set_active( (type+0) % 2 )
-        self.widget.show()
+        self.widget.show_all()
 
     def on_entry_activate(self, entry):
         for el in self.entrylists:
@@ -187,7 +186,7 @@ class ListWidget(gnomeglade.Component):
         model = self.treeview.get_model()
         pref = []
         for row in model:
-            pref.append("%s\t%s\t%s" % (row[0], row[1], row[2]))
+            pref.append("%s\t%s\t%s" % (row[0], row[1], misc.unescape(row[2])))
         setattr( self.prefs, self.key, "\n".join(pref) )
     def _update_filter_model(self):
         model = self.treeview.get_model()
@@ -197,7 +196,7 @@ class ListWidget(gnomeglade.Component):
             iter = model.append()
             model.set_value( iter, 0, filt.name)
             model.set_value( iter, 1, filt.active)
-            model.set_value( iter, 2, filt.value)
+            model.set_value( iter, 2, misc.escape(filt.value))
    
 ################################################################################
 #
@@ -254,7 +253,7 @@ class PreferencesDialog(gnomeglade.Component):
         cols = [ ("Name", type("")), ("Active", type(0)), ("Regex", type("")) ]
         self.textfilter = ListWidget( cols, self.prefs, "regexes")
         self.text_filters_box.pack_start(self.textfilter.widget)
-        self.checkbutton_ignore_blank_lines = self.prefs.ignore_blank_lines
+        self.checkbutton_ignore_blank_lines.set_active( self.prefs.ignore_blank_lines )
         # encoding
         self.entry_text_codecs.set_text( self.prefs.text_codecs )
         self._map_widgets_into_lists( ["save_encoding"] )
@@ -394,31 +393,6 @@ class NotebookLabel(gtk.HBox):
         if onclose:
             self.button.connect("clicked", onclose)
 
-
-
-################################################################################
-#
-# MeldNewMenu
-#
-################################################################################
-class MeldNewMenu(gnomeglade.Component):
-    def __init__(self, app):
-        gladefile = paths.share_dir("glade2/meldapp.glade")
-        gnomeglade.Component.__init__(self, gladefile, "popup_new")
-        self.parent = app
-    def on_menu_new_diff2_activate(self, *extra):
-        self.parent.on_menu_new_diff2_activate()
-    def on_menu_new_diff3_activate(self, *extra):
-        self.parent.on_menu_new_diff3_activate()
-    def on_menu_new_dir2_activate(self, *extra):
-        self.parent.on_menu_new_dir2_activate()
-    def on_menu_new_dir3_activate(self, *extra):
-        self.parent.on_menu_new_dir3_activate()
-    def on_menu_new_cvsview_activate(self, *extra):
-        self.parent.on_menu_new_cvsview_activate()
-    def on_menu_new_svnview_activate(self, *extra):
-        self.parent.on_menu_new_svnview_activate()
-
 ################################################################################
 #
 # MeldPreferences
@@ -426,6 +400,8 @@ class MeldNewMenu(gnomeglade.Component):
 ################################################################################
 class MeldPreferences(prefs.Preferences):
     defaults = {
+        "window_size_x": prefs.Value(prefs.INT, 600),
+        "window_size_y": prefs.Value(prefs.INT, 600),
         "use_custom_font": prefs.Value(prefs.BOOL,0),
         "custom_font": prefs.Value(prefs.STRING,"monospace, 14"),
         "tab_size": prefs.Value(prefs.INT, 4),
@@ -459,11 +435,12 @@ class MeldPreferences(prefs.Preferences):
         "color_edited_fg" : prefs.Value(prefs.STRING, "Black"),
         "filters" : prefs.Value(prefs.STRING, "Backups\t1\t#*# .#* ~* *~ *.{orig,bak,swp}\n" + \
                                               "CVS\t1\tCVS\n" + \
+                                              "SVN\t1\t.svn\n" + \
                                               "Binaries\t1\t*.{pyc,a,obj,o,so,la,lib,dll}\n" + \
                                               "Media\t0\t*.{jpg,gif,png,wav,mp3,ogg,xcf,xpm}"),
-        "regexes" : prefs.Value(prefs.STRING, "CVS keywords\t0\t\$[^:]+:[^\$]+\$\n" + \
+        "regexes" : prefs.Value(prefs.STRING, "CVS keywords\t0\t\$\\w+(:[^\\n$]+)?\$\n" + \
                                               "C++ comment\t0\t//.*\n" + \
-                                              "C comment\t0\t/\*[^*]*\*+([^/*][^*]*\*+)*/\n" + \
+                                              "C comment\t0\t/\*.*?\*/\n" + \
                                               "All whitespace\t0\t[ \\t\\r\\f\\v]*\n" + \
                                               "Leading whitespace\t0\t^[ \\t\\r\\f\\v]*\n" + \
                                               "Script comment\t0\t#.*"),
@@ -540,8 +517,7 @@ class MeldApp(gnomeglade.GnomeApp):
     def __init__(self):
         gladefile = paths.share_dir("glade2/meldapp.glade")
         gnomeglade.GnomeApp.__init__(self, "meld", version, gladefile, "meldapp")
-        self._map_widgets_into_lists( ["menu_file_save_file", "settings_drawstyle"] )
-        self.popup_new = MeldNewMenu(self)
+        self._map_widgets_into_lists( "settings_drawstyle".split() )
         self.statusbar = MeldStatusBar(self.task_progress, self.task_status, self.doc_status)
         self.prefs = MeldPreferences()
         if not developer:#hide magic testing button
@@ -554,6 +530,8 @@ class MeldApp(gnomeglade.GnomeApp):
         self.idle_hooked = 0
         self.scheduler = task.LifoScheduler()
         self.scheduler.connect("runnable", self.on_scheduler_runnable )
+        self.widget.set_default_size(self.prefs.window_size_x, self.prefs.window_size_y)
+        self.widget.show()
 
     def on_idle(self):
         ret = self.scheduler.iteration()
@@ -593,9 +571,6 @@ class MeldApp(gnomeglade.GnomeApp):
         newseq = newdoc.undosequence
         self.button_undo.set_sensitive(newseq.can_undo())
         self.button_redo.set_sensitive(newseq.can_redo())
-        for i in range(3):
-            sensitive = newdoc.num_panes > i
-            self.menu_file_save_file[i].set_sensitive(sensitive)
         nbl = self.notebook.get_tab_label( newdoc.widget )
         self.widget.set_title( nbl.label.get_text() + " - Meld")
         self.statusbar.set_doc_status("")
@@ -613,37 +588,19 @@ class MeldApp(gnomeglade.GnomeApp):
 
     def on_can_redo(self, undosequence, can):
         self.button_redo.set_sensitive(can)
-    
+
+    def on_size_allocate(self, window, rect):
+        self.prefs.window_size_x = rect.width
+        self.prefs.window_size_y = rect.height
+
     #
     # Toolbar and menu items (file)
     #
-    def on_menu_file_new_activate(self, *extra):
+    def on_menu_file_new_activate(self, menuitem):
         NewDocDialog(self, NewDocDialog.TYPE.DIFF2)
-    def on_menu_new_diff2_activate(self, *extra):
-        NewDocDialog(self, NewDocDialog.TYPE.DIFF2)
-    def on_menu_new_diff3_activate(self, *extra):
-        NewDocDialog(self, NewDocDialog.TYPE.DIFF3)
-    def on_menu_new_dir2_activate(self, *extra):
-        NewDocDialog(self, NewDocDialog.TYPE.DIR2)
-    def on_menu_new_dir3_activate(self, *extra):
-        NewDocDialog(self, NewDocDialog.TYPE.DIR3)
-    def on_menu_new_cvsview_activate(self, *extra):
-        NewDocDialog(self, NewDocDialog.TYPE.CVS)
-    def on_menu_new_svnview_activate(self, *extra):
-        NewDocDialog(self, NewDocDialog.TYPE.SVN)
 
     def on_menu_save_activate(self, menuitem):
-        try:
-            index = self.menu_file_save_file.index(menuitem)
-        except ValueError:
-            index = -1
-        try:
-            if index >= 0: # save one
-                self.current_doc().save_file(index)
-            else: # save all
-                self.current_doc().save()
-        except AttributeError:
-            pass
+        self.current_doc().save()
 
     def on_menu_refresh_activate(self, *args):
         self.current_doc().refresh()
@@ -712,7 +669,7 @@ class MeldApp(gnomeglade.GnomeApp):
         gnomeglade.url_show("http://bugzilla.gnome.org/buglist.cgi?product=meld")
 
     def on_menu_users_manual_activate(self, button):
-        gnomeglade.url_show("file:///"+os.path.abspath(paths.doc_dir("manual.html") ) )
+        gnomeglade.url_show("ghelp:///"+os.path.abspath(paths.help_dir("C/meld.xml") ))
 
     def on_menu_about_activate(self, *extra):
         about = gtk.glade.XML(paths.share_dir("glade2/meldapp.glade"),"about").get_widget("about")
@@ -729,14 +686,14 @@ class MeldApp(gnomeglade.GnomeApp):
             #self.append_filediff( ("ntest/file9a", "ntest/file9b") )
         pass
 
-    def on_menu_down_activate(self, *args):
+    def on_menu_edit_down_activate(self, *args):
         misc.safe_apply( self.current_doc(), "next_diff", gtk.gdk.SCROLL_DOWN )
 
-    def on_menu_up_activate(self, *args):
+    def on_menu_edit_up_activate(self, *args):
         misc.safe_apply( self.current_doc(), "next_diff", gtk.gdk.SCROLL_UP )
 
     def on_toolbar_new_clicked(self, *args):
-        self.popup_new.widget.popup(None, None, None, 3, gtk.get_current_event_time() )
+        NewDocDialog(self, NewDocDialog.TYPE.DIFF2)
 
     def on_toolbar_stop_clicked(self, *args):
         self.current_doc().stop()
@@ -785,13 +742,7 @@ class MeldApp(gnomeglade.GnomeApp):
         aredirs = [ os.path.isdir(p) for p in paths ]
         arefiles = [ os.path.isfile(p) for p in paths ]
         if (1 in aredirs) and (1 in arefiles):
-            m = _("Cannot compare a mixture of files and directories.\n")
-            for i in range(len(paths)):
-                status = aredirs[i] and _("folder") \
-                    or arefiles[i] and _("file") \
-                    or _("nonexistant")
-                m += "\n(%s)\t`%s'" % (status, paths[i])
-            misc.run_dialog( m,
+            misc.run_dialog( _("Cannot compare a mixture of files and directories.\n"),
                     parent = self,
                     buttonstype = gtk.BUTTONS_OK)
         elif 1 in aredirs:
@@ -852,9 +803,13 @@ usage_string = _("""Meld is a file and directory comparison tool. Usage:
     meld <file> <file> [file]   Start with 2 or 3 way file comparison
     meld <dir>  <dir>  [dir]    Start with 2 or 3 way directory comparison
 
+Options:
+    -h, --help                  Show this help text and exit
+    -v, --version               Display the version and exit
+
 For more information choose help -> contents.
 Report bugs at http://bugzilla.gnome.org/buglist.cgi?product=meld
-stevek@gnome.org
+Discuss meld at http://mail.gnome.org/mailman/listinfo/meld-list
 """)
 
 version_string = _("""Meld %s
@@ -875,17 +830,17 @@ def main():
         def __getattr__(self, attr):
             return getattr(self.file, attr)
     sys.stdout = Unbuffered(sys.stdout)
+    args = sys.argv[1:]
 
-    if len(sys.argv) == 2:
-        if sys.argv[1] in ("-h", "--help"):
+    if len(args) >= 2:
+        if "-h" in args or "--help" in args:
             print usage_string
             return
-        elif sys.argv[1] in ("-v", "--version"):
+        if "-v" in args or "--version" in args:
             print version_string
             return
 
     app = MeldApp()
-    args = sys.argv[1:]
 
     if len(args) == 0:
         pass
@@ -893,7 +848,10 @@ def main():
     elif len(args) == 1:
         a = args[0]
         if os.path.isfile(a):
-            doc = cvsview.CvsView(app.prefs)
+            if os.path.exists(os.path.join(os.path.dirname(a), '.svn')):
+                doc = svnview.CvsView(app.prefs)
+            else:
+                doc = cvsview.CvsView(app.prefs)
             def cleanup():
                 app.scheduler.remove_scheduler(doc.scheduler)
             app.scheduler.add_task(cleanup)
@@ -910,7 +868,7 @@ def main():
     elif len(args) in (2,3):
         app.append_diff(args)
     else:
-        app.usage( _("Wrong number of arguments (Got %i)") % len(arg))
+        app.usage( _("Wrong number of arguments (Got %i)") % len(args))
 
     app.main()
 
